from recosu.pso import global_best

# 1) COSU steps, definition of parameters to calibrate
steps = [
    {  # step 1
        "param": [
            {"name": "soilOutLPS", "bounds": (0.0, 2.0)},
            {"name": "lagInterflow", "bounds": (10.0, 80.0)},
        ],
        "objfunc": [
            {
                "name": "ns",  # Name (Must be unique)
                "of": "ns",  # Objective Function
                "data": (
                    "obs_data02_14.csv/obs/orun[1]",
                    "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                )
                # Optional Weight parameter. If not specified it is 1 or evenly distributed between all objective functions
            }
        ],
    },
    {  # step 2
        "param": [
            {"name": "flowRouteTA", "bounds": (0.4, 5.0)},
            {"name": "soilMaxDPS", "bounds": (0.0, 5.0)},
        ],
        "objfunc": [
            {
                "name": "ns",
                "of": "ns",
                "data": (
                    "obs_data02_14.csv/obs/orun[1]",
                    "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                ),
            }
        ],
    },
]

steps = [
    {
        "param": [
            {
                "name": "soilOutLPS",
                "bounds": (0.0, 2.0),
                "default_value": 1.0,
                "optimal_value": 0.0,
                "type": "float",
                "calibration_strategy": "none",
            },
            {
                "name": "lagInterflow",
                "bounds": (10.0, 80.0),
                "default_value": 1.0,
                "optimal_value": 0.0,
                "type": "float",
                "calibration_strategy": "none",
            },
        ],
        "objfunc": [
            {
                "name": "ns",
                "of": "ns",
                "weight": 1.0,
                "data": (
                    "obs_data02_14.csv/obs/orun[1]",
                    "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                ),
            }
        ],
    },
    {
        "param": [
            {
                "name": "flowRouteTA",
                "bounds": (0.4, 5.0),
                "default_value": 1.0,
                "optimal_value": 0.0,
                "type": "float",
                "calibration_strategy": "none",
            },
            {
                "name": "soilMaxDPS",
                "bounds": (0.0, 5.0),
                "default_value": 1.0,
                "optimal_value": 0.0,
                "type": "float",
                "calibration_strategy": "none",
            },
        ],
        "objfunc": [
            {
                "name": "ns",
                "of": "ns",
                "weight": 1.0,
                "data": (
                    "obs_data02_14.csv/obs/orun[1]",
                    "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                ),
            }
        ],
    },
]

# 2) static ages parameters
args = {
    "param": [
        {"name": "startTime", "value": "2002-01-01"},
        {"name": "endTime", "value": "2006-12-31"},
        {"name": "dataStartTime", "value": "2002-01-01"},
        {"name": "dataEndTime", "value": "2006-12-31"},
        {"name": "cal_startTime", "value": "2003-01-01"},
        {"name": "cal_endTime", "value": "2006-12-31"},
        {"name": "parallelismThreads", "value": "2"},
        {"name": "flagLoadState", "value": "True"},
        {"name": "payload", "value": "false"},
        {"name": "project", "value": "SFIR3"},
    ],
    # the service url
    "url": "http://csip.engr.colostate.edu:8087/csip-oms/m/ages/0.3.0",
    # the files to attach for each run
    "files": {},
}

args = {
    "param": [
        {"name": "startTime", "value": "2002-01-01"},
        {"name": "endTime", "value": "2006-12-31"},
        {"name": "dataStartTime", "value": "2002-01-01"},
        {"name": "dataEndTime", "value": "2006-12-31"},
        {"name": "cal_startTime", "value": "2003-01-01"},
        {"name": "cal_endTime", "value": "2006-12-31"},
        {"name": "parallelismThreads", "value": "2"},
        {"name": "flagLoadState", "value": "True"},
        {"name": "payload", "value": "false"},
        {"name": "project", "value": "SFIR3"},
    ],
    "url": "http://csip.engr.colostate.edu:8087/csip-oms/m/ages/0.3.0",
    "mode": "Optimization: MG-PSO",
    "files": {},
}

if "mode" in args:
    del args["mode"]

# 3. perform optimization
optimizer, trace = global_best(
    steps,  # step definition
    rounds=(1, 2),  # min/max number of rounds
    args=args,  # static arguments
    n_particles=10,  # number of particle candidates for each param
    iters=20,  # max # of iterations
    n_threads=10,  # number of threads to use
    # ftol=0.00000001,                          # min cost function delta for convergence
    options={"c1": 2, "c2": 2, "w": 0.8},  # hyperparameter
    oh_strategy={
        "w": "adaptive",
        "c1": "adaptive",
        "c2": "adaptive",
    },  # adaptive hyperparameter adjustments based on current and max # of iterations
    conf={
        "service_timeout": 400.0,
        "http_retry": 5,
        "http_allow_redirects": True,
        "async_call": False,
        "http_conn_timeout": 10,
        "http_read_timeout": 400,
        "particles_fail": 5,
        "step_trace": "./step_trace.json",
    },
)

step_trace = {
    "dir": "/Users/robertcordingly/Library/Python/3.8/lib/python/site-packages/mgpsogui/gui",
    "start": "2024-05-28 14:25:53.793407",
    "min_rounds": 1,
    "max_rounds": 2,
    "iters": 20,
    "ftol": "-inf",
    "ftol_iter": 1,
    "rtol": 0.001,
    "rtol_iter": 1,
    "n_threads": 10,
    "n_particles": 10,
    "n_steps": 2,
    "steps": [
        {
            "param": [
                {
                    "name": "soilOutLPS",
                    "bounds": (0.0, 2.0),
                    "default_value": 1.0,
                    "optimal_value": 0.0,
                    "type": "float",
                    "calibration_strategy": "none",
                },
                {
                    "name": "lagInterflow",
                    "bounds": (10.0, 80.0),
                    "default_value": 1.0,
                    "optimal_value": 0.0,
                    "type": "float",
                    "calibration_strategy": "none",
                },
            ],
            "objfunc": [
                {
                    "name": "ns",
                    "of": "ns",
                    "weight": 1.0,
                    "data": (
                        "obs_data02_14.csv/obs/orun[1]",
                        "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                    ),
                }
            ],
        },
        {
            "param": [
                {
                    "name": "flowRouteTA",
                    "bounds": (0.4, 5.0),
                    "default_value": 1.0,
                    "optimal_value": 0.0,
                    "type": "float",
                    "calibration_strategy": "none",
                },
                {
                    "name": "soilMaxDPS",
                    "bounds": (0.0, 5.0),
                    "default_value": 1.0,
                    "optimal_value": 0.0,
                    "type": "float",
                    "calibration_strategy": "none",
                },
            ],
            "objfunc": [
                {
                    "name": "ns",
                    "of": "ns",
                    "weight": 1.0,
                    "data": (
                        "obs_data02_14.csv/obs/orun[1]",
                        "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                    ),
                }
            ],
        },
    ],
    "args": "{'param': [{'name': 'startTime', 'value': '2002-01-01'}, {'name': 'endTime', 'value': '2006-12-31'}, {'name': 'dataStartTime', 'value': '2002-01-01'}, {'name': 'dataEndTime', 'value': '2006-12-31'}, {'name': 'cal_startTime', 'value': '2003-01-01'}, {'name': 'cal_endTime', 'value': '2006-12-31'}, {'name': 'parallelismThreads', 'value': '2'}, {'name': 'flagLoadState', 'value': 'True'}, {'name': 'payload', 'value': 'false'}, {'name': 'project', 'value': 'SFIR3'}], 'url': 'http://csip.engr.colostate.edu:8087/csip-oms/m/ages/0.3.0', 'files': {}}",
    "r1s1": {
        "time": "2024-05-28 14:28:27.648143",
        "best_costs": [0.4962544759194877, "inf"],
        "steps": [
            {
                "param": [
                    {
                        "name": "soilOutLPS",
                        "bounds": (0.0, 2.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.4277364777450963,
                    },
                    {
                        "name": "lagInterflow",
                        "bounds": (10.0, 80.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 65.31198215742819,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.4962544759194877,
            },
            {
                "param": [
                    {
                        "name": "flowRouteTA",
                        "bounds": (0.4, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                    },
                    {
                        "name": "soilMaxDPS",
                        "bounds": (0.0, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
            },
        ],
    },
    "r1s2": {
        "time": "2024-05-28 14:30:53.464613",
        "best_costs": [0.4962544759194877, 0.4159144385219078],
        "steps": [
            {
                "param": [
                    {
                        "name": "soilOutLPS",
                        "bounds": (0.0, 2.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.4277364777450963,
                    },
                    {
                        "name": "lagInterflow",
                        "bounds": (10.0, 80.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 65.31198215742819,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.4962544759194877,
            },
            {
                "param": [
                    {
                        "name": "flowRouteTA",
                        "bounds": (0.4, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.9846472555222765,
                    },
                    {
                        "name": "soilMaxDPS",
                        "bounds": (0.0, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 4.619363774024649,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.4159144385219078,
            },
        ],
    },
    "r1": {
        "time": "2024-05-28 14:30:53.465478",
        "round_cost": 0.9121689144413955,
        "best_costs": [0.4962544759194877, 0.4159144385219078],
        "improvements": [False, False],
    },
    "r2s1": {
        "time": "2024-05-28 14:33:23.457742",
        "best_costs": [0.41218845540212334, 0.4159144385219078],
        "steps": [
            {
                "param": [
                    {
                        "name": "soilOutLPS",
                        "bounds": (0.0, 2.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.3832527609622032,
                    },
                    {
                        "name": "lagInterflow",
                        "bounds": (10.0, 80.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 71.21736822883048,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.41218845540212334,
            },
            {
                "param": [
                    {
                        "name": "flowRouteTA",
                        "bounds": (0.4, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.9846472555222765,
                    },
                    {
                        "name": "soilMaxDPS",
                        "bounds": (0.0, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 4.619363774024649,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.4159144385219078,
            },
        ],
    },
    "r2s2": {
        "time": "2024-05-28 14:35:50.514486",
        "best_costs": [0.41218845540212334, 0.4121061580572347],
        "steps": [
            {
                "param": [
                    {
                        "name": "soilOutLPS",
                        "bounds": (0.0, 2.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.3832527609622032,
                    },
                    {
                        "name": "lagInterflow",
                        "bounds": (10.0, 80.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 71.21736822883048,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.41218845540212334,
            },
            {
                "param": [
                    {
                        "name": "flowRouteTA",
                        "bounds": (0.4, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 1.9461260670740506,
                    },
                    {
                        "name": "soilMaxDPS",
                        "bounds": (0.0, 5.0),
                        "default_value": 1.0,
                        "optimal_value": 0.0,
                        "type": "float",
                        "calibration_strategy": "none",
                        "value": 3.7734465992198514,
                    },
                ],
                "objfunc": [
                    {
                        "name": "ns",
                        "of": "ns",
                        "weight": 1.0,
                        "data": (
                            "obs_data02_14.csv/obs/orun[1]",
                            "output/csip_run/out/Outlet.csv/output/catchmentSimRunoff",
                        ),
                    }
                ],
                "cost": 0.4121061580572347,
            },
        ],
    },
    "r2": {
        "time": "2024-05-28 14:35:50.521162",
        "round_cost": 0.8242946134593581,
        "best_costs": [0.41218845540212334, 0.4121061580572347],
        "improvements": [False, False],
    },
    "rounds": 2,
    "end": "2024-05-28 14:35:51.017597",
    "time": "0:09:56.726582",
}



from customtkinter import CTkScrollableFrame
from customtkinter import CTkFrame
from customtkinter import CTkLabel
from customtkinter import CTkButton
from customtkinter import CTkEntry
import tkinter as tk
import subprocess
import platform
import os

global option_manager

class SamplingListView(CTkScrollableFrame):
    def __init__(self, *args,
                 option_manager: None,
                 step_index: 0,
                 bound_index: 0,
                 **kwargs):
        super().__init__(*args, **kwargs)
        
        self.option_manager = option_manager
        self.step_index = step_index
        self.bounds_index = bound_index
        self.key_values = []
        self.visual_name = tk.StringVar()
        self.visual_name.set("name")
        
        
        self.name = self.option_manager.get_steps()[self.step_index]["parameter_objects"][bound_index]["name"].get()
        self.default = self.option_manager.get_steps()[self.step_index]["parameter_objects"][bound_index]["default_value"].get()
        try:
            if self.name != "" and self.default != "":
                
                self.visual_name.set(self.name.rsplit('/', 1)[0] + '/')
                self.names = self.name.split('/')
                self.rows = self.names[-1].split(';')
                self.defaults = self.default.replace("[", "").replace("]", "").replace("(", "").replace(")", "").replace(" ", "").split(",")
                
                print("rows", self.rows)
                print("defaults", self.defaults)
                index = 0
                
                for name in self.rows:
                    obj = {"name": tk.StringVar(), "value": tk.StringVar()}
                    obj['name'].set(name)
                    obj['value'].set(self.defaults[index])
                    self.key_values.append(obj)
                    index += 1
        except Exception as e:
            print(e)
        
        if (len(self.key_values) == 0):
            self.add_key()
        
        self.edit_mode = False
        
        self.render()

    def clear(self):
        self.containerFrame.destroy()
        
    def toggle_edit_mode(self):
        self.clear()
        self.edit_mode = not self.edit_mode
        self.render()
        
    def add_key(self, key="0000", value="0"):
        obj = {"name": tk.StringVar(), "value": tk.StringVar()}
        obj['name'].set(key)
        obj['value'].set(value)
        self.key_values.append(obj)
        
    def remove_key(self, index):
        self.key_values.pop(index)
        
    def render(self):
        row = 0
        index = 0
        
        self.containerFrame = CTkFrame(self)
        self.containerFrame.grid(row=0, column=0, padx=(5, 5), pady=(5, 5), sticky="nsew")
        self.containerFrame.grid_columnconfigure((0, 1), weight=1)
        
        
        CTkLabel(self.containerFrame, text="Row:").grid(row=row, column=0, columnspan=1, padx=5, pady=5, sticky="")
        CTkLabel(self.containerFrame, text="Default:").grid(row=row, column=1, columnspan=1, padx=5, pady=5, sticky="")
        row += 1
        
        for key_value_pair in self.key_values:
            CTkEntry(self.containerFrame, textvariable=self.key_values[index]["name"]).grid(row=row, column=0, padx=(5, 5), pady=(5, 5), sticky="ew")
            
            if self.edit_mode:
                return_func = lambda index=index: (self.clear(), self.remove_key(index), self.render())
                CTkButton(self.containerFrame, text="Remove", command=return_func).grid(row=row, column=1, padx=(5, 5), pady=(5, 5), sticky="ew")
            else:
                bb = CTkEntry(self.containerFrame)
                bb.grid(row=row, column=1, padx=(5, 5), pady=(5, 5), sticky="ew")
                bb.configure(textvariable=self.key_values[index]["value"])
            row += 1
            index += 1
            
        if self.edit_mode:
            CTkButton(self.containerFrame, text="Exit", command=self.toggle_edit_mode).grid(row=row, column=0, padx=(5, 5), pady=(5, 5), sticky="ew")
        else:
            CTkButton(self.containerFrame, text="Edit", command=self.toggle_edit_mode).grid(row=row, column=0, padx=(5, 5), pady=(5, 5), sticky="ew")
            
        add_key_func = lambda: (self.clear(), self.add_key(), self.render())
        CTkButton(self.containerFrame, text="Add", command=add_key_func).grid(row=row, column=1, padx=(5, 5), pady=(5, 5), sticky="ew")
        
        row += 1
        
    def push_to_option_manager(self):
        visual_name = self.visual_name.get()
        default_values = "["
        first = True
        for key_value_pair in self.key_values:
            if first:
                visual_name += key_value_pair['name'].get()
                first = False
            else:
                visual_name += ";" + key_value_pair['name'].get()
            default_values += key_value_pair['value'].get() + ","
            
        default_values = default_values[:-1] + "]"
            
        self.option_manager.get_steps()[self.step_index]["parameter_objects"][self.bounds_index]["name"].set(visual_name)
        self.option_manager.get_steps()[self.step_index]["parameter_objects"][self.bounds_index]["default_value"].set(default_values)